#include <vector>
#include <iostream>
#include <fstream>
#include <cmath>


#ifdef OSX
#include <GLUT/glut.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#else
#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>
#endif

#include <time.h>
#include <math.h>


using namespace std;

bool SMOOTH_SHADING = false;


//****************************************************
// Some Classes
//****************************************************

class Camera;
class Viewport;
class Light;

class Light {
public:
	GLfloat ambient[4];		// ambient color, RGBA
	GLfloat diffuse[4];		// diffuse color, RGBA
	GLfloat specular[4];	// specular color, RGBA
	GLfloat pos[4];			// position XYZW
	GLenum id;				// light identifier
};

class Camera {
public:
	float eye[3];				// eye point, cop
	float center[3];			// point the eye is looking at
	float up[3];				// up direction
	float fov, near, far;		// field of view, near and far clipping planes
};

class Viewport {
public:
	int w, h; // width and height
};


//****************************************************
// Global Variables
//****************************************************
Camera		myCamera;
Light		myLight;
Viewport	viewport;




//****************************************************
// set up of basic scene (for my scene these things don't change)
//****************************************************
void initScene(){
 glClearColor(0.0,0.0,0.0,0.0);									// initial color of window
 glShadeModel(GL_SMOOTH);										// smooth shading
 glEnable(GL_LIGHTING);											// allow for lights
 glEnable(GL_DEPTH_TEST);										// enable depth testing; required for visibility tests
 
 
   // **** Set properties of the camera *****
  myCamera.eye[0]		= 0.0;	myCamera.eye[1]		= 2.0;	myCamera.eye[2]		= 3.0; 
  myCamera.center[0]	= 0.0;	myCamera.center[1]	= 2.0;	myCamera.center[2]	= 7.0; 
  myCamera.up[0]		= 0.0;	myCamera.up[1]		= 1.0;	myCamera.up[2]		= 0.0;
  
  myCamera.fov	= 60;			
  myCamera.near	= 0.1;
  myCamera.far	= 100.0;

  // **** Set properties of the light *****
  myLight.ambient[0]	= 0.15;	myLight.ambient[1]	= 0.15;	myLight.ambient[2]	= 0.15;	myLight.ambient[3]	= 0.15;// ambient color, RGBA
  myLight.diffuse[0]	= 0.75;	myLight.diffuse[1]	= 0.75;	myLight.diffuse[2]	= 0.75;	myLight.diffuse[3]	= 0.75;// diffuse color, RGBA
  myLight.specular[0]	= 0.65;	myLight.specular[1]	= 0.65;	myLight.specular[2]	= 0.65;	myLight.specular[3]	= 0.65;// specular color, RGBA
	
  myLight.pos[0] = 0.0;  myLight.pos[1] = 3.0; myLight.pos[2] = 8.0; myLight.pos[3] = 1.0;		// position to place light
  myLight.id=GL_LIGHT0+0;		// id for this light
 
 
}


//****************************************************
// reshape viewport if the window is resized
//****************************************************
void myReshape(int w, int h) {
	glViewport (0,0,w,h);
	viewport.w = w;
	viewport.h = h;
}

//****************************************************
// function that does the actual drawing of stuff
//***************************************************
void myDisplay() {
	
	float objBaseColor[3];											// base color of the object	
	float objThisLightTypeColor[3];									// color after multiplication with light properties
	float ks = 0.1,ka = 0.75,kd = 0.75;								// specular,ambient,and diffuse material properties
	
	objBaseColor[0] = .9;											// set the base color of the object (basically red)
	objBaseColor[1] = 0.0; 
	objBaseColor[2] = 0.0;	
	
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);				// clear the color and depth buffer
	
	//glDisable(GL_CULL_FACE);										// disable culling so that both sides of faces show up
	
	
	//***** Properties of Camera ******
	glMatrixMode(GL_PROJECTION);									// indicate we are specifying camera transformations
	glLoadIdentity();												// make sure camera transfomation is "zero'd"
	
	gluPerspective(myCamera.fov, ((float)viewport.w)/viewport.h,	// specify properties of camera
					myCamera.near, myCamera.far);						
					
	gluLookAt(myCamera.eye[0], myCamera.eye[1], myCamera.eye[2],			// specifies a tranformation such that myCamera is at eye and looking twords
			  myCamera.center[0], myCamera.center[1], myCamera.center[2],	// center (view direction is center minus eye) 
			  myCamera.up[0], myCamera.up[1], myCamera.up[2]);
	
	
	//**** Properties of Model *****
	glMatrixMode(GL_MODELVIEW);										// indicate we are specifying camera transformations
	glLoadIdentity();												// make sure transfomation is "zero'd"
	
	//**** Properties of Lights ***** 
	glLightfv(myLight.id, GL_AMBIENT, myLight.ambient);				// set ambient parameters
	glLightfv(myLight.id, GL_DIFFUSE, myLight.diffuse);				// set diffuse parameters
	glLightfv(myLight.id, GL_SPECULAR, myLight.specular);			// set specular parameters
	glLightfv(myLight.id, GL_POSITION, myLight.pos);				// set position
	glEnable(myLight.id);											// turn the light on	
	
	
	//**** Material Properties of Object ***** 
	// set ambient color
	objThisLightTypeColor[0] = ka*objBaseColor[0]; objThisLightTypeColor[1] = ka*objBaseColor[1];  objThisLightTypeColor[2] = ka*objBaseColor[2]; 
	glMaterialfv(GL_FRONT, GL_AMBIENT, objThisLightTypeColor);	// set the ambient material properties
	
	// set diffuse color
	objThisLightTypeColor[0] = kd*objBaseColor[0]; objThisLightTypeColor[1] = kd*objBaseColor[1];  objThisLightTypeColor[2] = kd*objBaseColor[2]; 
	glMaterialfv(GL_FRONT, GL_DIFFUSE, objThisLightTypeColor);	// set the diffuse material properties
	
	// set specular color
	objThisLightTypeColor[0] = ks*objBaseColor[0]; objThisLightTypeColor[1] = ks*objBaseColor[1];  objThisLightTypeColor[2] = ks*objBaseColor[2]; 
	glMaterialfv(GL_FRONT, GL_SPECULAR, objThisLightTypeColor);	// set the specular material properties
	
	
	//**** Draw Some Objects ****
	glPushMatrix();						// "remember where you are (how you've tranformed till now)"
	glTranslatef(0.0,2.0,9.0);			// translate to this position
	glutSolidSphere(1.0,20,16);			// draw a sphere
	
	glTranslatef(1.5,-0.5,0.0);			// translate starting from last place translated to 
	glRotatef(-90,1.0,0.0,0.0);			// rotate it (note that order applied is rotate then translate see documentation)
	glutSolidCone(1.0,2.0,20,16);		// draw a cone (note color unchanged, same material properties apply)
	glPopMatrix();						// "go to last place you remembered"


	//**** New Material Properties of Object ***** 
	// new base color
	objBaseColor[0] = .0;											// set the base color of the object (basically red)
	objBaseColor[1] = 0.0; 
	objBaseColor[2] = 0.9;	
	// set new material properties
	objThisLightTypeColor[0] = ka*objBaseColor[0]; objThisLightTypeColor[1] = ka*objBaseColor[1];  objThisLightTypeColor[2] = ka*objBaseColor[2]; 
	glMaterialfv(GL_FRONT, GL_AMBIENT, objThisLightTypeColor);	// set the ambient material properties
	objThisLightTypeColor[0] = kd*objBaseColor[0]; objThisLightTypeColor[1] = kd*objBaseColor[1];  objThisLightTypeColor[2] = kd*objBaseColor[2]; 
	glMaterialfv(GL_FRONT, GL_DIFFUSE, objThisLightTypeColor);	// set the diffuse material properties
	objThisLightTypeColor[0] = ks*objBaseColor[0]; objThisLightTypeColor[1] = ks*objBaseColor[1];  objThisLightTypeColor[2] = ks*objBaseColor[2]; 
	glMaterialfv(GL_FRONT, GL_SPECULAR, objThisLightTypeColor);	// set the specular material properties


	//**** Draw Some More Objects ****
	glPushMatrix();						// "remember where you are"
	glTranslatef(-1.5,2.0,9.0);			// translate from place before previous push matrix
	glRotatef(45.0,0.0,0.0,1.0);		// rotate it (note that order applied is rotate then translate see documentation)
	glutSolidCube(1.0);					// draw a cube
	glPopMatrix();						// "go to last place you remembered"



	glutSwapBuffers();					// swap buffers (we earlier set double buffer)
	
	
}


//****************************************************
// timer function so that the clock is updated at least every second
//****************************************************
void myTimerFunc(int id) {
	glutPostRedisplay();
	glutTimerFunc(1000, myTimerFunc, 0);
}


//****************************************************
// the usual stuff, nothing exciting here
//****************************************************
int main(int argc, char *argv[]) {
  //This initializes glut
  glutInit(&argc, argv);
  
  //This tells glut to use a double-buffered window with red, green, and
  //blue channels and a depth buffer.
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

  //The size and position of the window
  glutInitWindowSize(400, 400);
  glutInitWindowPosition(0,0);
  glutCreateWindow(argv[0]);


  initScene();										// quick function to set up scene
  


  glutDisplayFunc(myDisplay);						// function to run when its time to draw something
  glutReshapeFunc(myReshape);						// function to run when the window gets resized
  //glutKeyboardFunc(myKeyboard);					
  //glutSpecialFunc(mySpecial);
  glutTimerFunc(1000, myTimerFunc, 0);

  glutMainLoop();									// infinite loop that will keep drawing and resizing and whatever else
  
  return 0;
}








